import React, { useState, useRef, useEffect, Children } from 'react';
import {
    SelectContainer,
    SelectButton,
    SelectList,
    SelectHead,
    SelectLabel,
    SelectMain,
    LeftArrowContainer,
    RightArrowContainer,
    ErrorContainer,
} from './NsSelect.styled';
import { NsIcon } from '../icon';
import { SelectProps } from './types.d';
const NsSelect: React.FC<SelectProps> = ({
    selectedValue,
    selectedLabel,
    onSelect,
    children,
    disabled = false,
    type = 'outlined',
    placeholder = '',
    selectBorderColor = 'accentColorB',
    closeOnMouseLeave = false,
    showListDivider = true,
    label,
    readOnly = false,
    errorMessage,
    hasArrows,
    // eslint-disable-next-line no-unused-vars
    rules,
    reverseArrowControls,
}) => {
    const [isOpen, setIsOpen] = useState(false);
    const [value, setValue] = useState(selectedValue || '');
    const [selectedIndex, setSelectedIndex] = useState(-1);
    const selectContainerRef = useRef<HTMLDivElement>(null);
    const selectHeadRef = useRef<HTMLDivElement>(null);

    const closeSelect = () => {
        if (closeOnMouseLeave) {
            setIsOpen(false);
        }
    };

    const handleSelect = (input: string, index: number) => {
        onSelect(input);
        setIsOpen(false);
        setValue(input);
        setSelectedIndex(index);
    };

    // eslint-disable-next-line react-hooks/exhaustive-deps
    const handleClickOutside = (event: MouseEvent) => {
        if (selectContainerRef.current && !selectContainerRef.current.contains(event.target as Node) && isOpen) {
            setIsOpen(false);
        }
    };

    const handleMouseClick = () => {
        if (disabled) return;
        setIsOpen(!isOpen);
        if (!isOpen) {
            setTimeout(() => {
                selectHeadRef.current?.focus();
            }, 0);
        }
    };
    const handleArrowRightClick = () => {
        if (selectedIndex !== -1) {
            const nextIndex = reverseArrowControls ? selectedIndex - 1 : selectedIndex + 1;
            const childrenArray = Children.toArray(children);
            const optionAtIndex = childrenArray[nextIndex];
            const { value } = (optionAtIndex as any)?.props ?? {};
            handleSelect(value, nextIndex);
        } else {
            if (!reverseArrowControls) {
                const nextIndex = selectedIndex + 1;
                const childrenArray = Children.toArray(children);
                const optionAtIndex = childrenArray[nextIndex];
                const { value } = (optionAtIndex as any)?.props ?? {};
                handleSelect(value, nextIndex);
            }
        }
    };
    const handleArrowLeftClick = () => {
        if (selectedIndex !== -1) {
            const nextIndex = reverseArrowControls ? selectedIndex + 1 : selectedIndex - 1;
            const childrenArray = Children.toArray(children);
            const optionAtIndex = childrenArray[nextIndex];
            const { value } = (optionAtIndex as any)?.props ?? {};
            handleSelect(value, nextIndex);
        } else {
            if (reverseArrowControls) {
                const nextIndex = selectedIndex + 1;
                const childrenArray = Children.toArray(children);
                const optionAtIndex = childrenArray[nextIndex];
                const { value } = (optionAtIndex as any)?.props ?? {};
                handleSelect(value, nextIndex);
            }
        }
    };

    useEffect(() => {
        document.addEventListener('click', handleClickOutside);
        return () => {
            document.removeEventListener('click', handleClickOutside);
        };
    }, [handleClickOutside, isOpen]);

    return (
        <SelectMain>
            {hasArrows && (
                <LeftArrowContainer
                    onClick={handleArrowLeftClick}
                    className="LeftArrow"
                    style={{
                        visibility:
                            !reverseArrowControls && (selectedIndex === 0 || selectedIndex < 0) ? 'hidden' : 'visible',
                    }}
                >
                    <NsIcon name="arrowLeft" color="primary" />
                </LeftArrowContainer>
            )}
            <SelectContainer
                data-testid="select-container"
                disabled={disabled || readOnly}
                ref={selectContainerRef}
                errorMessage={errorMessage}
            >
                {label && <SelectLabel errorMessage={errorMessage}>{label}</SelectLabel>}
                <SelectHead
                    data-testid="select-head"
                    disabled={disabled || readOnly}
                    type={type}
                    onMouseDown={handleMouseClick}
                    tabIndex={0}
                    ref={selectHeadRef}
                >
                    <SelectButton>{selectedLabel || placeholder}</SelectButton>
                    {!readOnly && <NsIcon name={isOpen ? 'arrowUp' : 'arrowDown'} color="primary" />}
                </SelectHead>
                {isOpen ? (
                    <SelectList
                        data-testid="select-list"
                        onMouseLeave={closeSelect}
                        borderColor={selectBorderColor}
                        className="selectList"
                    >
                        {React.Children.map(children, (child, index) => {
                            if (React.isValidElement(child)) {
                                return React.cloneElement(child, {
                                    // @ts-ignore
                                    onSelect: (input: string) => handleSelect(input, index),
                                    selectedValue: value,
                                    showListDivider,
                                    index,
                                });
                            }
                            return null;
                        })}
                    </SelectList>
                ) : null}
                {errorMessage && <ErrorContainer>{errorMessage}</ErrorContainer>}
            </SelectContainer>
            {hasArrows && (
                <RightArrowContainer
                    onClick={handleArrowRightClick}
                    className="RightArrow"
                    style={{
                        visibility:
                            reverseArrowControls && (selectedIndex === 0 || selectedIndex < 0) ? 'hidden' : 'visible',
                    }}
                >
                    <NsIcon name="arrowRight" color="primary" />
                </RightArrowContainer>
            )}
        </SelectMain>
    );
};

export default NsSelect;
